//	Copyright (c) 2008 itok ( http://itok.jp/ , http://110k.net/ )
//	All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without modification, 
//	are permitted provided that the following conditions are met:
//
//	- Redistributions of source code must retain the above copyright notice, 
//	  this list of conditions and the following disclaimer.
//	- Redistributions in binary form must reproduce the above copyright notice, 
//	  this list of conditions and the following disclaimer in the documentation 
//	  and/or other materials provided with the distribution.
//	- Neither the name of itok nor the names of its contributors may be used to endorse 
//	  or promote products derived from this software without specific prior written permission.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS 
//	AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, 
//	INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
//	AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
//	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
//	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
//	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
//	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//	HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
//	STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
//	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import "PUService.h"
#import "KeychainAccessor.h"

@implementation PUService

-(id) init
{
	if (self = [super init]) {
		m_activated = NO;
	}
	return self;
}

-(void) dealloc
{
	if (m_usernameKey) {
		[m_usernameKey release];
	}
	if (m_keychainURL) {
		[m_keychainURL release];
	}
	[super dealloc];
}

-(void) setupAccount
{
	NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
	NSString* username = [defaults objectForKey:m_usernameKey];
	[self setValue:username forKey:@"username"];
	if (!IS_NULL_STR(_username)) {
		NSString* password = [KeychainAccessor getInternetPassword:username url:m_keychainURL];
		[self setValue:password forKey:@"password"];
	}	
}

-(void) setUsername:(id)value
{
	BOOL changed = YES;
	if (_username) {
		if ([value isEqualTo:_username]) {
			changed = NO;
		}
		[_username release];
	}
	_username = [value copy];
	
	if (changed) {
		[KeychainAccessor changeInternetPassword:_password account:_username url:m_keychainURL];
		
		NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
		[defaults setValue:_username forKey:m_usernameKey];
		[defaults synchronize];
		
		if (m_activated) {
			[self login];
		}
	}
}

-(void) setPassword:(id)value
{
	BOOL changed = YES;
	if (_password) {
		if ([value isEqualTo:_password]) {
			changed = NO;
		}
		[_password release];
	}
	_password = [value copy];
	
	if (changed) {
		[KeychainAccessor changeInternetPassword:_password account:_username url:m_keychainURL];
		
		if (m_activated) {
			[self login];
		}
	}
}

#pragma mark ___ PUService protocol ___

-(NSView*) view
{
	return o_view;
}

-(NSString*) title
{
	return @"";
}

-(void) activate
{
	[self login];
	m_activated = YES;
}

-(void) deactivate
{
	m_activated = NO;
}

-(NSArray*) requestsWithMgr:(ExportMgr*)mgr andURLs:(NSDictionary*)urlDic
{
	return nil;
}

-(NSDictionary*) convertAttributes
{
	return nil;
}

-(NSURL*) URLForLastUpload:(NSArray*)responses
{
	return nil;
}

-(void) login
{
	return;
}

-(BOOL) checkAuth
{
	return (!(IS_NULL_STR([self valueForKey:@"username"]) || IS_NULL_STR([self valueForKey:@"password"])));
}

@end
