//	Copyright (c) 2008 itok ( http://itok.jp/ , http://110k.net/ )
//	All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without modification, 
//	are permitted provided that the following conditions are met:
//
//	- Redistributions of source code must retain the above copyright notice, 
//	  this list of conditions and the following disclaimer.
//	- Redistributions in binary form must reproduce the above copyright notice, 
//	  this list of conditions and the following disclaimer in the documentation 
//	  and/or other materials provided with the distribution.
//	- Neither the name of itok nor the names of its contributors may be used to endorse 
//	  or promote products derived from this software without specific prior written permission.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS 
//	AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, 
//	INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
//	AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
//	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
//	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
//	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
//	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
//	HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
//	STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
//	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#import "iPhotoUploader.h"
#import "PUMacros.h"
#import "PUMultiRequest.h"

#import "PhotoZou.h"
#import "Fotolife.h"

@implementation iPhotoUploader

-(IBAction) cancelUpload:(id)sender
{
	if (m_mreq) {
		[m_mreq cancel];
	}
}

#pragma mark ___ delegate ___

-(void) tabView:(NSTabView*)tabView didSelectTabViewItem:(NSTabViewItem*)tabViewItem
{
	m_curService = [m_services objectForKey:[tabViewItem identifier]];
	if (m_curService) {
		[m_curService activate];
		
		NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
		[defaults setObject:[tabViewItem identifier] forKey:@"CurrentService"];
		[defaults synchronize];
	}
}

-(void) multiRequestDidCompleted:(PUMultiRequest*)multiReq result:(int)result
{
	[NSApp endSheet:o_prgWindow];
	[o_prgWindow orderOut:nil];
	
	[o_prgBar stopAnimation:nil];

	switch (result) {
		case kPUMultiRequest_Success :
		{
			NSURL* url = [m_curService URLForLastUpload:m_resArr];
			if (url) {
				int ret = NSRunAlertPanel(LOCALIZED(@"Upload Completed"), LOCALIZED(@"browse uploaded photos?"), @"Yes", @"No", @"");
				if (ret == NSAlertDefaultReturn) {
					[[NSWorkspace sharedWorkspace] openURL:url];
				}
			} else {
				NSRunAlertPanel(LOCALIZED(@"Upload Completed"), @"", @"OK", @"", @"");
			}
		}
			break;
		case kPUMultiRequest_Cancel :
			NSRunAlertPanel(LOCALIZED(@"Upload Canceled"), @"", @"OK", @"", @"");
			break;
		case kPUMultiRequest_Fail :
			NSRunAlertPanel(LOCALIZED(@"Upload Failed"), @"", @"OK", @"", @"");
			break;
	}
			
	[[m_exMgr exportController] cancel:nil];
	
	[m_resArr removeAllObjects];
	[multiReq release];
	m_mreq = nil;
}

-(void) multiRequest:(PUMultiRequest*)multiReq didCompletedOne:(PURequest*)request withProgress:(double)progress
{
	if ([o_prgBar doubleValue] == 0) {
		[o_prgBar setIndeterminate:NO];
	} else if (progress == 1.0) {
		[o_prgBar setIndeterminate:YES];
	}
	[o_prgBar setDoubleValue:progress];
	
	id res = [request responseData];
	if (res) {
		[m_resArr addObject:res];
	}
}

-(BOOL) multiRequest:(PUMultiRequest*)multiReq willStartOne:(PURequest*)request
{
	return YES;
}

#pragma mark ___ ExportPluginProtocol ___

-(id) initWithExportImageObj:(id)fp8
{
	if (self = [super init]) {
		m_exMgr = fp8;
		[NSBundle loadNibNamed:@"iPhotoUploader" owner:self];
		m_services = nil;
		m_resArr = [[NSMutableArray alloc] init];
		
		NSString* about = [NSString stringWithFormat:@"v%@ Copyright (c) 2008 itok (http://itok.jp/)", [[[NSBundle bundleWithIdentifier:BUNDLE_IDENTIFIER] infoDictionary] objectForKey:@"CFBundleVersion"]];
		[self setValue:about forKey:@"about"];
	}
	return self;
}

-(void) dealloc
{
	[m_services release];
	[m_resArr release];
	
	[super dealloc];
}

-(id) settingsView
{
	return o_box;
}

-(id) firstView
{
	return o_box;
}

-(void) viewWillBeActivated
{
	if (!m_services) {
		while ([o_tab numberOfTabViewItems]) {
			[o_tab removeTabViewItem:[o_tab tabViewItemAtIndex:0]];
		}

		static const char* service_names[] = {
			"PhotoZouService",
			"FotolifeService",
			"FlickrService",
			nil
		};
		int i;
		NSMutableDictionary* services = [NSMutableDictionary dictionary];
		for (i = 0; ; i++) {
			const char* name = service_names[i];
			if (!name) {
				break;
			}
			
			id<PUService> service = [[NSClassFromString([NSString stringWithCString:name]) alloc] init];
			NSTabViewItem* item = [[NSTabViewItem alloc] initWithIdentifier:[service title]];
			[item setView:[service view]];
			[item setLabel:[service title]];
			[o_tab addTabViewItem:item];
			[services setObject:service forKey:[service title]];
			[item release];
			[(NSObject*)service release];
		}
		m_services = [[NSDictionary alloc] initWithDictionary:services];
		
		NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
		id identifier = [defaults objectForKey:@"CurrentService"];
		if (identifier) {
			[o_tab selectTabViewItemWithIdentifier:identifier];
		}
	}
	
	m_curService = [m_services objectForKey:[[o_tab selectedTabViewItem] identifier]];
	if (m_curService) {
		[m_curService activate];
	}
}

-(void) viewWillBeDeactivated
{
}

-(id) requiredFileType
{
	return @"";
}

-(BOOL) wantsDestinationPrompt
{
	return NO;
}

-(id) getDestinationPath
{
	return @"";
}

-(id) defaultFileName
{
	return @"";
}

-(id) defaultDirectory
{
	return NSHomeDirectory();
}

-(BOOL) treatSingleSelectionDifferently
{
	return NO;
}

-(BOOL) handlesMovieFiles
{
	return NO;
}

-(BOOL) validateUserCreatedPath:(id)fp8
{
	return YES;
}

-(void) clickExport
{
}

-(void) startExport:(id)fp8
{
	[m_resArr removeAllObjects];
	
	if (![m_curService checkAuth]) {
		NSRunAlertPanel(LOCALIZED(@"Please enter username and password"), @"", @"OK", @"", @"");
		return;
	}
	
	NSArray* reqs = [m_curService requestsWithMgr:m_exMgr andURLs:nil];
	if (reqs && [reqs count] > 0) {
		[o_prgBar setMaxValue:1.0];
		[o_prgBar setMinValue:0.0];
		[o_prgBar setDoubleValue:0.0];
		[o_prgBar setUsesThreadedAnimation:YES];
		[o_prgBar setIndeterminate:YES];
		[o_prgBar startAnimation:nil];
		[NSApp beginSheet:o_prgWindow modalForWindow:[o_box window] modalDelegate:nil didEndSelector:nil contextInfo:nil];		
		
		m_mreq = [[PUMultiRequest alloc] init];
		[m_mreq setDelegate:self];
		[m_mreq start:reqs];
	}
}

-(void) performExport:(id)fp8
{
}

-(void*) progress
{
	return (void*)@"";
}

-(void) lockProgress
{
}

-(void) unlockProgress
{
}

-(void) cancelExport
{
}

-(id) name
{
	return LOCALIZED(@"iPhotoUploader");
}

@end
